#module wide variables

from .convert import mol2weight, weight2mol, get_molar_mass, molar_mass, formula2components

if __name__ == "__main__":
    print("Library to do simple conversions in glass science.")
